#ifndef OGSE_FXAA_H
#include "common.h"
#define OGSE_FXAA_H
float3 calc_fxaa( float3 img, float2 tc )
{
	float FXAA_SPAN_MAX = 4.0;
	float FXAA_REDUCE_MUL = 1.0/16.0;
	float FXAA_REDUCE_MIN = 1.0/256.0;

	float3 rgbNW = tex2D(s_image, tc + (float2(-1.0,-1.0)*common_params.zw)).xyz;
	float3 rgbNE = tex2D(s_image, tc + (float2(1.0,-1.0)*common_params.zw)).xyz;
	float3 rgbSW = tex2D(s_image, tc + (float2(-1.0,1.0)*common_params.zw)).xyz;
	float3 rgbSE = tex2D(s_image, tc + (float2(1.0,1.0)*common_params.zw)).xyz;
	float3 rgbM = tex2D(s_image, tc).xyz;
   
	float3 luma = float3(0.299, 0.587, 0.114);
	float lumaNW = dot(rgbNW, luma);
    float lumaNE = dot(rgbNE, luma);
    float lumaSW = dot(rgbSW, luma);
    float lumaSE = dot(rgbSE, luma);
    float lumaM  = dot(rgbM,  luma);
   
	float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
    float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));
   
	float2 dir;
    dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
    dir.y =  ((lumaNW + lumaSW) - (lumaNE + lumaSE));
   
	float dirReduce = max(
        (lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * FXAA_REDUCE_MUL),
        FXAA_REDUCE_MIN);
      
	float rcpDirMin = 1.0/(min(abs(dir.x), abs(dir.y)) + dirReduce);
   
	dir = min(float2( FXAA_SPAN_MAX,  FXAA_SPAN_MAX),
          max(float2(-FXAA_SPAN_MAX, -FXAA_SPAN_MAX),
          dir * rcpDirMin)) * common_params.zw;
       
	float3 rgbA = (1.0/2.0) * (
        tex2D(s_image, tc.xy + dir * (1.0/3.0 - 0.5)).xyz +
        tex2D(s_image, tc.xy + dir * (2.0/3.0 - 0.5)).xyz);
	float3 rgbB = rgbA * (1.0/2.0) + (1.0/4.0) * (
        tex2D(s_image, tc.xy + dir * (0.0/3.0 - 0.5)).xyz +
        tex2D(s_image, tc.xy + dir * (3.0/3.0 - 0.5)).xyz);
	float lumaB = dot(rgbB, luma);
	float3 final = rgbB;
	if((lumaB < lumaMin) || (lumaB > lumaMax)){
		final = rgbA;
	}
	return (final - img);
}
#endif